@@ -86,7 +86,7 @@ class mapView(ranaModule):
         if(self.setCentre(pos)):
           self.set('centreOnce', False)
         self.lastPos = pos
-
+        
 #    request = self.get("centreOn", None)
 #    if(request):
 #      self.setCentre([float(a) for a in request.split(",")])
@@ -94,22 +94,22 @@ class mapView(ranaModule):
   def setCentre(self,pos):
     """takes care for centering the map on current position"""
     proj = self.m.get('projection', None)
-    if(proj == None):
-      return(False)
-    if(pos == None):
-      return(False)
-    
-    (lat,lon) = pos
-    self.set('map_centre', pos)
-
-    z = int(self.get('z', 15))
-    x,y = latlon2xy(lat,lon,z)
-
-    if(not self.d.has_key('viewport')):
-      return(False)
-    (sx,sy,sw,sh) = self.get('viewport')
-    proj.setView(sx,sy,sw,sh)
-    proj.recentre(lat,lon,z)
-    proj.setZoom(z)
-    self.set("needRedraw", True)
-    return(True)
\ No newline at end of file
+    if proj and pos:
+      (lat,lon) = pos
+      self.set('map_centre', pos)
+      z = int(self.get('z', 15))
+      x,y = proj.ll2xy(lat,lon)
+      if(not self.d.has_key('viewport')):
+        return(False)
+      (sx,sy,sw,sh) = self.get('viewport')
+      """
+      the shift amount represents a perentage of the distance from screen center
+      to an edge:
+      ceneter+---------->|edge
+      0 -> no shift
+      1 -> directly on the edge
+      0.5 -> shifted by half of this distance, eq. in 3/4 of the screen
+      """
+      proj.recentre(lat,lon,z)
+      self.set("needRedraw", True)
+      return(True)
\ No newline at end of file
