@@ -44,10 +44,14 @@ class menus(ranaModule):
     self.notificationModule = None
     self.hideMapSreenButtons = False
     self.lastHideCheckTimestamp = time.time()
+    self.itemMenuGrid = (None,[])
 
     # colors - failsafe defaults
     self.mainTextColor = (0,0,0.3,1)
     self.centerButtonCircleColor = (0,0,1,0.45)
+    self.spButtonFillTup = ("#ffec8b",1.0)
+    self.spButtonOutlineTup = ("#8b814c",1.0)
+
 
   def update(self):
     # check if buttons should be hidden, every second by default
@@ -123,8 +127,6 @@ class menus(ranaModule):
             dx = size * min(w,h)
             dy = dx
 
-
-
           for key in buttons:
             if key in item:
               (px,py,ndx,ndy) = [float(i) for i in item[key]]
@@ -135,7 +137,10 @@ class menus(ranaModule):
 
 
 
-      menuIcon = self.get('mode', 'car')
+      modeIcon = self.get('mode', 'car')
+      # use thinner outline and smaller corner radius
+      menuIcon = "%s>generic:;;;;5;10" % modeIcon
+
 
       (x1,y1) = buttons['zoom_out']
       self.drawButton(cr, x1, y1, dx, dy, '', minusIcon, "mapView:zoomOut")
@@ -230,39 +235,6 @@ class menus(ranaModule):
     else:
       cr.move_to(x,y)
       pg.show_layout(layout)
-
-## this ca be used when I can get it to behave like the old show_text based method
-#  def drawText(self,cr,text,x,y,w,h,border=0):
-#    """pango based text drawing method"""
-#    if(not text):
-#      return
-#    # Put a border around the area
-#    if(border != 0):
-#      x += w * border
-#      y += h * border
-#      w *= (1-2*border)
-#      h *= (1-2*border)
-#    pg = pangocairo.CairoContext(cr)
-#    # create a layout for your drawing area
-#    layout = pg.create_layout()
-#    layout.set_markup(text)
-#    layout.set_font_description(pango.FontDescription("Sans Serif 60"))
-#    (lw,lh) = layout.get_size()
-#    if lw == 0 or lh == 0:
-#      return # no need to draw this + avoid a division by zero
-#    scale = float(pango.SCALE)
-##    factor = min(((w)/(lw/scale)),((h)/(lh/scale)))
-#    factorW = (w)/(lw/scale)
-#    factorH = (h)/(lh/scale)
-##    factor = min(factor, 1.0)
-#    factor = min(factorW, factorH)
-#    cr.move_to(x,y)
-#    cr.save()
-#    cr.scale(factor,factor)
-#    ratio = max(lw / w, lh / h)
-#    cr.set_font_size(60 / ratio)
-#    pg.show_layout(layout)
-#    cr.restore()
     
   def drawText(self,cr,text,x,y,w,h,border=0):
     """This is mainly used to draw the text on icons,
@@ -317,7 +289,6 @@ class menus(ranaModule):
     self.icons.draw(cr,icon,x1,y1,w,h)
 
     # Draw text
-#    cr.set_source_rgb(0, 0, 0.3)
     cr.set_source_rgba(*self.mainTextColor)
     if text != None:
       textList = text.split('#')
@@ -345,7 +316,16 @@ class menus(ranaModule):
     if(list != None):
       self.listOffset += dy
       print "Drag in menu + %f = %f" % (dy,self.listOffset)
-    
+
+  def setItemMenuGrid(self, x1, y1, cols,rows,dx,dy):
+    """generate an icon placement grid for a given number of
+       number of columns,rows and icon sizes"""
+    grid = []
+    for y in range(rows):
+      for x in range(cols):
+        grid.append((x1+x*dx, y1+y*dy))
+    self.itemMenuGrid = ((x1, y1, cols,rows,dx,dy),grid)
+
   def drawMenu(self, cr, menuName):
     """Draw menus"""
 #    print "current menu is:%s" % menuName
@@ -354,14 +334,6 @@ class menus(ranaModule):
       return
     (x1,y1,w,h) = self.get('viewport', None)
 
-#    list = self.lists.get(menuName, None)
-#    if(list != None):
-#      m = self.m.get(list, None)
-#      if(m != None):
-#        listHelper = listable_menu(cr,x1,y1,w,h, self.m.get('clickHandler', None), self.listOffset)
-#        m.drawList(cr, menuName, listHelper)
-#      return
-
     # Is it a list ?
     if menuName in self.lists.keys(): # TODO: optimize this
 #      print "drawing list: %s" % menuName
@@ -399,35 +371,76 @@ class menus(ranaModule):
     dx = w / cols
     dy = h / rows
 
+    # check if we have valid precomputed icon grid:
+    if self.itemMenuGrid[0] != (x1, y1, cols,rows,dx,dy):
+      self.setItemMenuGrid(x1, y1, cols, rows, dx, dy)
+
     # for each item in the menu
     id = 0
-    for y in range(rows):
-      for x in range(cols):
-        item = menu.get(id, None)
-        if(item == None):
-          # menu drawing is done, do the master overlay hook
-          if self.notificationModule:
-            self.notificationModule.drawMasterOverlay(cr)
-          return
+    itemSlots = cols*rows
+    itemCount = menu['metadata']['itemCount']
+    if itemCount > itemSlots: # does this menu have more items than available slots
+      # get current page
+      pageNumber = menu['metadata']['currentPage']
+      # get colors for the more and less buttons
+      (spbFillColor,spbFillAlpha) = self.spButtonFillTup
+      (spbOutlineColor,spbOutlineAlpha) = self.spButtonOutlineTup
+      bgIconDescription = "generic:%s;%f;%s;%f;;" % (spbFillColor, spbFillAlpha, spbOutlineColor, spbOutlineAlpha)
+      if pageNumber == 0:
+        # just draw the "more" button
+        (x,y) = self.itemMenuGrid[1][-1]
+        self.drawButton(cr, x, y, dx, dy, "more", "more>%s" % bgIconDescription, "ml:menu:setIMPage:%s;%d|set:needRedraw:True" % (menuName, pageNumber+1))
+        itemGrid = self.itemMenuGrid[1][0:-1]
+      else:
+        id = (pageNumber)*(itemSlots-2)+1
+        if (itemCount-id-2) < (itemSlots-1):
+          # this is tle last page
+          # draw only the "less" button
+          (x,y) = self.itemMenuGrid[1][0]
+          self.drawButton(cr, x, y, dx, dy, "less", "less>%s" % bgIconDescription, "ml:menu:setIMPage:%s;%d|set:needRedraw:True" % (menuName, pageNumber-1))
+          itemGrid = self.itemMenuGrid[1][1:(itemCount-id-1)]
+        else:
+          # this is an intermediate page
+          # draw the "less" and "more" buttons
+          (x,y) = self.itemMenuGrid[1][0]
+          self.drawButton(cr, x, y, dx, dy, "less", "less>%s" % bgIconDescription, "ml:menu:setIMPage:%s;%d|set:needRedraw:True" % (menuName, pageNumber-1))
+          (x,y) = self.itemMenuGrid[1][-1]
+          self.drawButton(cr, x, y, dx, dy, "more", "more>%s" % bgIconDescription, "ml:menu:setIMPage:%s;%d|set:needRedraw:True" % (menuName, pageNumber+1))
+          itemGrid = self.itemMenuGrid[1][1:-1]
+          # the first page only has one slot less, du to having only the "more" button
+    else:
+      # there are less items than slots
+      itemGrid = self.itemMenuGrid[1][0:itemCount]
 
-        # Draw it
-        type = item[3]
-        if type=='simple':
-          (text, icon, action, type) = item
-          self.drawButton(cr, x1+x*dx, y1+y*dy, dx, dy, text, icon, action)
-        elif type=='toggle':
-          index = item[1]
-          toggleCount = len(item[0])
-          nextIndex = (index + 1)%toggleCount
-          # like this, text and corresponding actions can be written on a single line
-          # eq: "save every 3 s", "nice icon", "set:saveInterval:3s"
-          text = item[0][index][0]
-          icon = item[0][nextIndex][1]
-          action = item[0][nextIndex][2]
+    for xy in itemGrid:
+      item = menu.get(id, None)
+      # Draw it
 
-          action+='|menu:toggle#%s#%s|set:needRedraw:True' % (menuName,id)
-          self.drawButton(cr, x1+x*dx, y1+y*dy, dx, dy, text, icon, action)
-        id += 1
+      # get coordinates
+      (x,y) = xy
+
+      type = item[3]
+      if type=='simple':
+        (text, icon, action, type) = item
+        self.drawButton(cr, x, y, dx, dy, text, icon, action)
+      elif type=='toggle':
+        index = item[1]
+        toggleCount = len(item[0])
+        nextIndex = (index + 1)%toggleCount
+        # like this, text and corresponding actions can be written on a single line
+        # eq: "save every 3 s", "nice icon", "set:saveInterval:3s"
+        text = item[0][index][0]
+        icon = item[0][nextIndex][1]
+        action = item[0][nextIndex][2]
+
+        action+='|menu:toggle#%s#%s|set:needRedraw:True' % (menuName,id)
+        self.drawButton(cr, x, y, dx, dy, text, icon, action)
+      id += 1
+
+    # menu drawing is done, do the master overlay hook
+    if self.notificationModule:
+      self.notificationModule.drawMasterOverlay(cr)
+    return
 
   def register(self, menu, type, module):
     """Register a menu as being handled by some other module"""
@@ -435,36 +448,32 @@ class menus(ranaModule):
       self.lists[menu] = module
     else:
       print "Can't register \"%s\" menu - unknown type" % type
+
+  def initMenu(self,menu):
+    """anitialize menu a menu dictionary instance to default parameters"""
+    self.menus[menu] = {'metadata':{'itemCount':0,'currentPage':0}}
     
-  def clearMenu(self, menu, cancelButton='set:menu:main'):
-    self.menus[menu] = {}
-    if(cancelButton != None):
-      self.addItem(menu,'','up', cancelButton, 0)
+  def clearMenu(self, menu, cancelButtonAction='set:menu:main'):
+    self.initMenu(menu)
+    self.addItem(menu,'','up', cancelButtonAction, 0)
 
   def addItem(self, menu, text, icon=None, action=None, pos=None):
-    i = 0
-    while(pos == None):
-      if(self.menus[menu].get(i, None) == None):
-        pos = i
-      i += 1
-      if(i > 20):
-        print "Menu full, can't add %s" % text
+    if menu not in self.menus:
+      self.initMenu(menu)
+    itemCount = self.menus[menu]['metadata']['itemCount']
     type = "simple"
-    self.menus[menu][pos] = (text, icon, action, type)
-
+    """we are counting up from zero for the item indexes"""
+    self.menus[menu][itemCount] = (text, icon, action, type)
+    self.menus[menu]['metadata']['itemCount'] = itemCount + 1
 
   def addToggleItem(self, menu, textIconAction, index=0, pos=None, uniqueName=None):
     """
     add a togglable item to the menu
     textIconAction is a list of texts icons and actions -> (text,icon,action)
     """
-    i = 0
-    while(pos == None):
-      if(self.menus[menu].get(i, None) == None):
-        pos = i
-      i += 1
-      if(i > 20):
-        print "Menu full, can't add %s" % text
+    if menu not in self.menus:
+      self.initMenu(menu)
+    itemCount = self.menus[menu]['metadata']['itemCount']
     type = 'toggle'
     if uniqueName:
       perzist = self.get('persistentToggleButtons', None)
@@ -478,15 +487,17 @@ class menus(ranaModule):
         perzist[uniqueName] = index
         self.set('persistentToggleButtons', perzist)
 
-    self.menus[menu][pos] = (textIconAction, index, uniqueName, type)
+    """we are counting up from zero for the item indexes"""
+    self.menus[menu][itemCount] = (textIconAction, index, uniqueName, type)
+    self.menus[menu]['metadata']['itemCount'] = itemCount + 1
 
 
   def addListableMenu(self, name, items, parrentAction, descFunction=None, drawFunction=None):
     newListableMenu = self.listableMenu(name, self, items, parrentAction, descFunction, drawFunction,4)
     self.lists[name] = newListableMenu
     return newListableMenu
-      
 
+  
   class listableMenu:
     """a listable menu object"""
     def __init__(self, name, menus, items, parrentAction, descFunction=None, drawFunction=None, displayedItems=3):
@@ -616,7 +627,7 @@ class menus(ranaModule):
       self.addItem(
         'transport',                       # menu
         label,                             # label
-        label.lower(),                     # icon
+        mode,                     # icon
         'set:mode:'+mode+"|set:menu:None") # action
 
   def setupSearchWhereMenu(self):
@@ -745,7 +756,6 @@ class menus(ranaModule):
   def setupDataSubMenu(self, nextMenu='zoomDown', prevMenu='data'):
     """here we set the radius for download"""
     self.clearMenu('data2', "set:menu:%s" % prevMenu)
-#    self.addItem('data2', '5 km', 'generic', 'set:downloadSize:4|mapData:download|set:menu:editBatch')
     self.addItem('data2', '1 km', 'generic', 'set:downloadSize:1|set:menu:%s' % nextMenu)
     self.addItem('data2', '2 km', 'generic', 'set:downloadSize:2|set:menu:%s' % nextMenu)
     self.addItem('data2', '5 km', 'generic', 'set:downloadSize:4|set:menu:%s' % nextMenu)
@@ -760,7 +770,6 @@ class menus(ranaModule):
     """we can download tiles around "here" (GPS cooridnates), route or the current view"""
     self.clearMenu('data', "set:menu:%s" % prevMenu)
     self.addItem('data', 'Around here', 'generic', 'set:downloadType:data|set:downloadArea:here|set:menu:%s' % nextMenu)
-#    self.addItem('data', 'Around route', 'generic', 'set:downloadType:data|set:downloadArea:route|set:menu:%s' % nextMenu)
     self.addItem('data', 'Around route', 'generic', 'set:downloadType:data|set:downloadArea:route|set:menu:chooseRouteForDl')
     self.addItem('data', 'Around view', 'generic', 'set:downloadType:data|set:downloadArea:view|set:menu:%s' % nextMenu)
     self.setupDataSubMenu()
@@ -779,17 +788,12 @@ class menus(ranaModule):
 
   def setupGeneralMenus(self):
     self.clearMenu('main', "set:menu:None")
-    #self.addItem('main', 'map', 'generic', 'set:menu:layers')
-#    self.addItem('main', 'places', 'city', 'set:menu:placenames_categories')
-#    self.addItem('main', 'waypoints', 'waypoints', 'set:menu:waypoints_categories')
     self.addItem('main', 'route', 'route', 'set:menu:route')
     self.addItem('main', 'POI', 'poi', 'set:menu:poi')
     self.addItem('main', 'search', 'search', 'set:menu:searchWhere')
-    #self.addItem('main', 'view', 'view', 'set:menu:view')
     self.addItem('main', 'options', 'options', 'set:menu:options')
     self.addItem('main', 'download', 'download', 'set:menu:data')
     self.addItem('main', 'mode', 'mode', 'set:menu:transport')
-#    self.addItem('main', 'centre', 'centre', 'toggle:centred|set:menu:None')
     self.addItem('main', 'tracklogs', 'tracklogs', 'set:menu:tracklogManagerCathegories')
     self.addItem('main', 'log a track', 'log', 'set:menu:tracklog')
     self.setupTransportMenu()
@@ -800,7 +804,6 @@ class menus(ranaModule):
     self.setupDataMenu()
     self.setupRouteMenu()
     self.clearMenu('options', "set:menu:main") # will be filled by mod_options
-#    self.clearMenu('routeProfile', "set:menu:main") # will be filled by mod_routeProfile
     self.lists['places'] = 'placenames'
 
     self.set('editBatchMenuActive', False) # at startup, the edit batch menu is inactive
@@ -1085,6 +1088,10 @@ class menus(ranaModule):
   def colorsChangedCallback(self,colors):
     self.mainTextColor = colors['main_text'].getCairoColor()
     self.centerButtonCircleColor = colors['center_button_circle'].getCairoColor()
+    sbFill = colors['special_button__fill']
+    self.spButtonFillTup = (sbFill.getColorString(),sbFill.getAlpha())
+    sbOutline = colors['special_button__outline']
+    self.spButtonOutlineTup = (sbOutline.getColorString(),sbOutline.getAlpha())
 
   def firstTime(self):
     self.set("menu",None)
@@ -1112,7 +1119,10 @@ class menus(ranaModule):
           self.lists[listMenuName].scrollUp()
         elif args[1]=="down":
           self.lists[listMenuName].scrollDown()
-
+    elif (message == "setIMPage"):
+      menuName = args[0]
+      targetPageNr = int(args[1])
+      self.menus[menuName]['metadata']['currentPage'] = targetPageNr
     elif (message == "rebootDataMenu"):
       self.setupDataMenu() # we are returning from the batch menu, data menu needs to be "rebooted"
       self.set('editBatchMenuActive', False)
@@ -1142,7 +1152,6 @@ class menus(ranaModule):
       self.set(uniqueName, newIndex)
       self.menus[menu][pos] = (textIconAction, newIndex, uniqueName, type)
 
-    
 if(__name__ == "__main__"):
   a = menus({},{'viewport':(0,0,600,800)})
   #a.drawMapOverlay(None)
