@@ -336,9 +336,9 @@ class showPOI(ranaModule):
       setup the category chooser menu,
       and make sure the POI is commited after a cetegory is chosen
       """
-      self.sendMessage('ml:notification:Select a category for this POI:2')
       self.sendMessage('ml:showPOI:setupPOICategoryChooser:showPOI;setCatAndCommit')
       self.set('menu', 'POICategoryChooser')
+      self.sendMessage('ml:notification:Select a category for this POI:3')
 
     # "current position as a new POI" entry chain
     elif key=='newCurrentPositionName':
@@ -347,9 +347,12 @@ class showPOI(ranaModule):
     elif key=='newCurrentPositionDescription':
       self.activePOI.setDescription(result,commit=False)
       # setting the category is the last step
-      self.sendMessage('ml:notification:Select a category for this POI:2')
       self.sendMessage('ml:showPOI:setupPOICategoryChooser:showPOI;setCatAndCommit')
       self.set('menu', 'POICategoryChooser')
+      print "ADASDASDASD"
+      self.sendMessage('ml:notification:Select a category for this POI:3')
+      self.set('needRedraw', True)
+      print "ADASDASDASD"
 
 if(__name__ == "__main__"):
   a = example({}, {})
