@@ -218,7 +218,13 @@ class storePOI(ranaModule):
         menus.addItem('POIDetailTools', 'longitude#edit', 'generic', 'ms:showPOI:editActivePOI:lon')
         menus.addItem('POIDetailTools', 'category#change', 'generic', 'ml:showPOI:setupPOICategoryChooser:showPOI;setCatAndCommit|set:menu:POICategoryChooser')
         menus.addItem('POIDetailTools', 'position#set as', 'generic', 'showPOI:centerOnActivePOI|ml:gpsd:setPosLatLon:%f;%f' % (self.lat,self.lon))
-        menus.addItem('POIDetailTools', 'POI#delete', 'generic', 'showPOI:askDeleteActivePOI')
+        """just after the point is stored and and its detail menu shows up for the first time,
+        it cant be deleted from the database, beucause we dont know which index it got :D
+        TODO: find a free index and then store the point on it
+        (make sure noone writes to the database between getting the free index and writing the poi to it)
+        then we would be able to delete even newly created points"""
+        if self.getId():
+          menus.addItem('POIDetailTools', 'POI#delete', 'generic', 'showPOI:askDeleteActivePOI')
 
     def getValues(self):
       return([self.id,self.lat,self.lon,self.label,self.description,self.categoryId])
